@@ -35,21 +35,25 @@ int http_response_write_header(server *srv, connection *con) {
 	b = chunkqueue_get_prepend_buffer(con->write_queue);
 
 	if (con->request.http_version == HTTP_VERSION_1_1) {
-		BUFFER_COPY_STRING_CONST(b, "HTTP/1.1 ");
+		buffer_copy_string_len(b, CONST_STR_LEN("HTTP/1.1 "));
 	} else {
-		BUFFER_COPY_STRING_CONST(b, "HTTP/1.0 ");
+		buffer_copy_string_len(b, CONST_STR_LEN("HTTP/1.0 "));
 	}
 	buffer_append_long(b, con->http_status);
-	BUFFER_APPEND_STRING_CONST(b, " ");
+	buffer_append_string_len(b, CONST_STR_LEN(" "));
 	buffer_append_string(b, get_http_status_name(con->http_status));
 
 	if (con->request.http_version != HTTP_VERSION_1_1 || con->keep_alive == 0) {
-		BUFFER_APPEND_STRING_CONST(b, "\r\nConnection: ");
-		buffer_append_string(b, con->keep_alive ? "keep-alive" : "close");
+		buffer_append_string_len(b, CONST_STR_LEN("\r\nConnection: "));
+		if (con->keep_alive) {
+			buffer_append_string_len(b, CONST_STR_LEN("keep-alive"));
+		} else {
+			buffer_append_string_len(b, CONST_STR_LEN("close"));
+		}
 	}
 
 	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
-		BUFFER_APPEND_STRING_CONST(b, "\r\nTransfer-Encoding: chunked");
+		buffer_append_string_len(b, CONST_STR_LEN("\r\nTransfer-Encoding: chunked"));
 	}
 
 
@@ -60,14 +64,15 @@ int http_response_write_header(server *srv, connection *con) {
 		ds = (data_string *)con->response.headers->data[i];
 
 		if (ds->value->used && ds->key->used &&
-		    0 != strncmp(ds->key->ptr, "X-LIGHTTPD-", sizeof("X-LIGHTTPD-") - 1) &&
-			0 != strncmp(ds->key->ptr, "X-Sendfile", sizeof("X-Sendfile") - 1)) {
-			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Date"))) have_date = 1;
-			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Server"))) have_server = 1;
+		    0 != strncasecmp(ds->key->ptr, CONST_STR_LEN("X-LIGHTTPD-")) &&
+			0 != strcasecmp(ds->key->ptr, "X-Sendfile")) {
+			if (0 == strcasecmp(ds->key->ptr, "Date")) have_date = 1;
+			if (0 == strcasecmp(ds->key->ptr, "Server")) have_server = 1;
+			if (0 == strcasecmp(ds->key->ptr, "Content-Encoding") && 304 == con->http_status) continue;
 
-			BUFFER_APPEND_STRING_CONST(b, "\r\n");
+			buffer_append_string_len(b, CONST_STR_LEN("\r\n"));
 			buffer_append_string_buffer(b, ds->key);
-			BUFFER_APPEND_STRING_CONST(b, ": ");
+			buffer_append_string_len(b, CONST_STR_LEN(": "));
 #if 0
 			/** 
 			 * the value might contain newlines, encode them with at least one white-space
@@ -81,7 +86,7 @@ int http_response_write_header(server *srv, connection *con) {
 
 	if (!have_date) {
 		/* HTTP/1.1 requires a Date: header */
-		BUFFER_APPEND_STRING_CONST(b, "\r\nDate: ");
+		buffer_append_string_len(b, CONST_STR_LEN("\r\nDate: "));
 
 		/* cache the generated timestamp */
 		if (srv->cur_ts != srv->last_generated_date_ts) {
@@ -100,14 +105,14 @@ int http_response_write_header(server *srv, connection *con) {
 
 	if (!have_server) {
 		if (buffer_is_empty(con->conf.server_tag)) {
-			BUFFER_APPEND_STRING_CONST(b, "\r\nServer: " PACKAGE_NAME "/" PACKAGE_VERSION);
-		} else {
-			BUFFER_APPEND_STRING_CONST(b, "\r\nServer: ");
+			buffer_append_string_len(b, CONST_STR_LEN("\r\nServer: " PACKAGE_NAME "/" PACKAGE_VERSION));
+		} else if (con->conf.server_tag->used > 1) {
+			buffer_append_string_len(b, CONST_STR_LEN("\r\nServer: "));
 			buffer_append_string_encoded(b, CONST_BUF_LEN(con->conf.server_tag), ENCODING_HTTP_HEADER);
 		}
 	}
 
-	BUFFER_APPEND_STRING_CONST(b, "\r\n\r\n");
+	buffer_append_string_len(b, CONST_STR_LEN("\r\n\r\n"));
 
 
 	con->bytes_header = b->used - 1;
@@ -152,7 +157,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		 *  */
 
 		config_cond_cache_reset(srv, con);
-		config_setup_connection(srv, con); // Perhaps this could be removed at other places.
+		config_setup_connection(srv, con); /* Perhaps this could be removed at other places. */
 
 		if (con->conf.log_condition_handling) {
 			log_error_write(srv, __FILE__, __LINE__,  "s",  "run condition");
@@ -181,7 +186,11 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		 *
 		 */
 
-		buffer_copy_string(con->uri.scheme, con->conf.is_ssl ? "https" : "http");
+		if (con->conf.is_ssl) {
+			buffer_copy_string_len(con->uri.scheme, CONST_STR_LEN("https"));
+		} else {
+			buffer_copy_string_len(con->uri.scheme, CONST_STR_LEN("http"));
+		}
 		buffer_copy_string_buffer(con->uri.authority, con->request.http_host);
 		buffer_to_lower(con->uri.authority);
 
@@ -224,6 +233,29 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		}
 
 
+		/* build filename
+		 *
+		 * - decode url-encodings  (e.g. %20 -> ' ')
+		 * - remove path-modifiers (e.g. /../)
+		 */
+
+
+		if (con->request.http_method == HTTP_METHOD_OPTIONS &&
+		    con->uri.path_raw->ptr[0] == '*' && con->uri.path_raw->ptr[1] == '\0') {
+			/* OPTIONS * ... */
+			buffer_copy_string_buffer(con->uri.path, con->uri.path_raw);
+		} else {
+			buffer_copy_string_buffer(srv->tmp_buf, con->uri.path_raw);
+			buffer_urldecode_path(srv->tmp_buf);
+			buffer_path_simplify(con->uri.path, srv->tmp_buf);
+		}
+
+		if (con->conf.log_request_handling) {
+			log_error_write(srv, __FILE__, __LINE__,  "s",  "-- sanatising URI");
+			log_error_write(srv, __FILE__, __LINE__,  "sb", "URI-path     : ", con->uri.path);
+		}
+
+
 		/**
 		 *
 		 * call plugins
@@ -245,29 +277,6 @@ handler_t http_response_prepare(server *srv, connection *con) {
 			break;
 		}
 
-		/* build filename
-		 *
-		 * - decode url-encodings  (e.g. %20 -> ' ')
-		 * - remove path-modifiers (e.g. /../)
-		 */
-
-
-
-		if (con->request.http_method == HTTP_METHOD_OPTIONS &&
-		    con->uri.path_raw->ptr[0] == '*' && con->uri.path_raw->ptr[1] == '\0') {
-			/* OPTIONS * ... */
-			buffer_copy_string_buffer(con->uri.path, con->uri.path_raw);
-		} else {
-			buffer_copy_string_buffer(srv->tmp_buf, con->uri.path_raw);
-			buffer_urldecode_path(srv->tmp_buf);
-			buffer_path_simplify(con->uri.path, srv->tmp_buf);
-		}
-
-		if (con->conf.log_request_handling) {
-			log_error_write(srv, __FILE__, __LINE__,  "s",  "-- sanatising URI");
-			log_error_write(srv, __FILE__, __LINE__,  "sb", "URI-path     : ", con->uri.path);
-		}
-
 		/**
 		 *
 		 * call plugins
